// OIP.js

// Obtain viewer canvas object and its context.

var vcanvas = document.getElementById("vcanvas");
var vcontext = vcanvas.getContext('2d');

// Render a solid black viewer canvas -- black is the default fill.

vcontext.fillRect(0, 0, WIDTH, HEIGHT);

// Create original image canvas object and obtain its context.

var oicanvas = document.createElement("canvas");
var oicontext = oicanvas.getContext('2d');

// Create processed image canvas object and obtain its context.

var picanvas = document.createElement("canvas");
var picontext = picanvas.getContext('2d');

var image = new Image();

var dragging;
var dragox;
var dragoy;
var dragdx;
var dragdy;
var prevdragdx;
var prevdragdy;

// Load selected image from first entry in files sequence. When the image is
// loaded, handleReaderLoadEnd() is called to assign the image src to
// image.src. This ultimately results in the function assigned to image.onload
// executing, which initializes variables for performing picture movement on
// the viewer canvas, and renders the loaded image on the original image,
// processed image, and viewer canvases.

function loadImage(files)
{
   vcanvas.onmousemove = null;
   vcanvas.onmousedown = null;
   vcanvas.onmouseup = null;

   if (getSafariVersion() == null && getOperaVersion() == null)
   {
      var file = document.getElementById('input').files[0];
      var reader = new FileReader();
      reader.onloadend = handleReaderLoadEnd;
      reader.readAsDataURL(file);
   }
   else
   {
      image.src = filepath;
   }

   image.onload = function()
   {
      if (typeof(filename) != "undefined")
      {
         var img = new Image();
         img.src = "http://tutortutor.ca/cgi-bin/OIP_kf.php?file="+filename;
      }

      dragging = false;
      dragox = 0;
      dragoy = 0;
      dragdx = 0;
      dragdy = 0;
      prevdragdx = 0;
      prevdragdy = 0;

      vcanvas.onmousemove = doMouseMove;
      vcanvas.onmousedown = doMouseDown;
      vcanvas.onmouseup = doMouseUp;
      document.onmouseup = doMouseUp;

      oicanvas.width = image.width;
      oicanvas.height = image.height;
      oicontext.drawImage(image, 0, 0);

      picanvas.width = image.width;
      picanvas.height = image.height;
      picontext.drawImage(image, 0, 0);

      drawImage();
   }
}

// Convert image to grayscale.

function ctgs()
{
   var id = picontext.getImageData(0, 0, image.width, image.height);
   for (var i = 0; i < id.data.length; i += 4)
   {
      var val = id.data[i]*0.3+id.data[i+1]*0.59+id.data[i+2]*0.11;
      id.data[i] = val;
      id.data[i+1] = val;
      id.data[i+2] = val;
   }
   picontext.putImageData(id, 0, 0);
   drawImage();
}

// Handle mouse movement events by dragging image over viewer canvas if
// dragging is in effect.

function doMouseMove(evt)
{
   if (dragging)
   {
      if (image.width > WIDTH)
      {
         dragdx = evt.screenX-dragox+prevdragdx;
         if (dragdx < 0)
            dragdx = 0;
         else
         if (dragdx+WIDTH > image.width)
            dragdx = image.width-WIDTH;
      }
      if (image.height > HEIGHT)
      {
         dragdy = evt.screenY-dragoy+prevdragdy;
         if (dragdy < 0)
            dragdy = 0;
         else
         if (dragdy+HEIGHT > image.height)
            dragdy = image.height-HEIGHT;
      }
      drawImage();
   }
}

// Handle mouse button pressed events by initiating a drag operation if image
// does not completely fit on visible portion of viewer canvas.

function doMouseDown(evt)
{
   if (image.width < WIDTH && image.height < HEIGHT)
      return;
     
   dragging = true;
   dragox = evt.screenX;
   dragoy = evt.screenY;
}

// Handle mouse button released events to terminate a drag operation.

function doMouseUp(evt)
{
   dragging = false;
   prevdragdx = dragdx;
   prevdragdy = dragdy;
}

// Draw image on viewer canvas.

function drawImage()
{
   // Must clear viewer canvas. If canvas not cleared, if a larger image is
   // displayed, and if a smaller image is then displayed, parts of the larger
   // image will appear.

   vcontext.fillRect(0, 0, WIDTH, HEIGHT);
   vcontext.drawImage(picanvas,
                      dragdx,
                      dragdy,
                      Math.min(image.width, WIDTH),
                      Math.min(image.height, HEIGHT),
                      (image.width < WIDTH) ? (WIDTH-image.width)/2 : 0,
                      (image.height < HEIGHT) ? (HEIGHT-image.height)/2 : 0,
                      Math.min(image.width, WIDTH),
                      Math.min(image.height, HEIGHT));
}

// Respond to the end of the image-load operation that was initiated in
// loadImage(files).

function handleReaderLoadEnd(event)
{
   image.src = event.target.result;
}

// Process image by converting to grayscale or sepia toning image, or undo all
// image-processing operations.

function process()
{
   var op = document.getElementById('processor_list').value;
   if (op == "ctgs")
      ctgs();
   else
   if (op == "st")
      st();
   else
      undo();
}

// Save canvas contents to file with randomly generated filename.

function saveToFile()
{
   var strData = picanvas.toDataURL("image/png");
   document.location.href = strData.replace("image/png",
                                            "image/octet-stream");
}

// Sepia tone image.

function st()
{
   ctgs();
   var id = picontext.getImageData(0, 0, image.width, image.height);

   for (var i = 0; i < id.data.length; i += 4)
   {
      id.data[i] = Math.min(id.data[i]+40, 255);
      id.data[i+1] = Math.min(id.data[i+1]+20, 255);
      id.data[i+2] = Math.max(id.data[i+2]-20, 0);
   }
   picontext.putImageData(id, 0, 0);
   drawImage();
}

// Undo all image-processing operations.

function undo()
{
   var id = oicontext.getImageData(0, 0, image.width, image.height);
   picontext.putImageData(id, 0, 0);
   drawImage();
}
